@@ -8,6 +8,8 @@ Hacked 2001 by drt@un.bewaff.net
 # TODO: unify assert / FilIf usage
 
 import sys
+import threading
+sys.path.append('.')
 sys.path.append('..')
 
 import IPy
@@ -17,44 +19,44 @@ import random
 testloops = 250
 
 class parseAddress(unittest.TestCase):
-    okValues = [('FEDC:BA98:7654:3210:FEDC:BA98:7654:3210', 338770000845734292534325025077361652240L),
-                ('FEDCBA9876543210FEDCBA9876543210', 338770000845734292534325025077361652240L),
-                ('0xFEDCBA9876543210FEDCBA9876543210', 338770000845734292534325025077361652240L),
-                ('1080:0000:0000:0000:0008:0800:200C:417A', 21932261930451111902915077091070067066L),
-                ('1080:0:0:0:8:800:200C:417A', 21932261930451111902915077091070067066L),
-                ('1080:0::8:800:200C:417A', 21932261930451111902915077091070067066L),
-                ('1080::8:800:200C:417A', 21932261930451111902915077091070067066L),
-                ('FF01:0:0:0:0:0:0:43', 338958331222012082418099330867817087043L),
-                ('FF01:0:0::0:0:43', 338958331222012082418099330867817087043L),
-                ('FF01::43', 338958331222012082418099330867817087043L),
-                ('0:0:0:0:0:0:0:1', 1L),
-                ('0:0:0::0:0:1', 1L),
-                ('::1', 1L),
-                ('0:0:0:0:0:0:0:0', 0L),
-                ('0:0:0::0:0:0', 0L),
-                ('::', 0L),
-                ('0:0:0:0:0:0:13.1.68.3', 218186755L),
-                ('::13.1.68.3', 218186755L),
-                ('0:0:0:0:0:FFFF:129.144.52.38', 281472855454758L),
-                ('::FFFF:129.144.52.38', 281472855454758L),
-                ('1080:0:0:0:8:800:200C:417A', 21932261930451111902915077091070067066L),
-                ('1080::8:800:200C:417A', 21932261930451111902915077091070067066L),
-                ('0.0.0.0', 0L),
-                ('0', 0L),
-                ('127.0.0.1', 2130706433L),
-                ('255.255.255.255', 4294967295L),
-                ('0.0.0.1', 1L),
-                ('1', 16777216L),
-                ('213.221.113.87', 3588059479L),
-                ('0000', 0L),
-                ('127001', 127001L),
-                ('1234576', 1234576L),
-                ('1', 16777216L),
-                ('232111387', 232111387L),
-                ('255', 4278190080L),
-                ('256', 256L),
-                ('0xffffffff', 4294967295L),
-                ('0x100000000', 4294967296L),
+    okValues = [('FEDC:BA98:7654:3210:FEDC:BA98:7654:3210', 338770000845734292534325025077361652240),
+                ('FEDCBA9876543210FEDCBA9876543210', 338770000845734292534325025077361652240),
+                ('0xFEDCBA9876543210FEDCBA9876543210', 338770000845734292534325025077361652240),
+                ('1080:0000:0000:0000:0008:0800:200C:417A', 21932261930451111902915077091070067066),
+                ('1080:0:0:0:8:800:200C:417A', 21932261930451111902915077091070067066),
+                ('1080:0::8:800:200C:417A', 21932261930451111902915077091070067066),
+                ('1080::8:800:200C:417A', 21932261930451111902915077091070067066),
+                ('FF01:0:0:0:0:0:0:43', 338958331222012082418099330867817087043),
+                ('FF01:0:0::0:0:43', 338958331222012082418099330867817087043),
+                ('FF01::43', 338958331222012082418099330867817087043),
+                ('0:0:0:0:0:0:0:1', 1),
+                ('0:0:0::0:0:1', 1),
+                ('::1', 1),
+                ('0:0:0:0:0:0:0:0', 0),
+                ('0:0:0::0:0:0', 0),
+                ('::', 0),
+                ('0:0:0:0:0:0:13.1.68.3', 218186755),
+                ('::13.1.68.3', 218186755),
+                ('0:0:0:0:0:FFFF:129.144.52.38', 281472855454758),
+                ('::FFFF:129.144.52.38', 281472855454758),
+                ('1080:0:0:0:8:800:200C:417A', 21932261930451111902915077091070067066),
+                ('1080::8:800:200C:417A', 21932261930451111902915077091070067066),
+                ('0.0.0.0', 0),
+                ('0', 0),
+                ('127.0.0.1', 2130706433),
+                ('255.255.255.255', 4294967295),
+                ('0.0.0.1', 1),
+                ('1', 16777216),
+                ('213.221.113.87', 3588059479),
+                ('0000', 0),
+                ('127001', 127001),
+                ('1234576', 1234576),
+                ('1', 16777216),
+                ('232111387', 232111387),
+                ('255', 4278190080),
+                ('256', 256),
+                ('0xffffffff', 4294967295),
+                ('0x100000000', 4294967296),
                 ('0xffffffffffffffffffffffffffffffff', 0xffffffffffffffffffffffffffffffffL),
                 ('0xdeadbeef', 0xdeadbeefL),
                 ('0xdeadbabe', 0xdeadbabeL),
@@ -63,8 +65,8 @@ class parseAddress(unittest.TestCase):
                 ('0xc0debabe', 0xc0debabeL),
                 ('0xbabec0de', 0xbabec0deL),
                 ('0xcafebabe', 0xcafebabeL),
-                ('0x1', 1L),
-                ('0xabcdef', 11259375L)]
+                ('0x1', 1),
+                ('0xabcdef', 11259375)]
 
     # TODO: check for more invalid input
 
@@ -81,7 +83,7 @@ class parseAddress(unittest.TestCase):
         self.assertEqual(version, 4)
         (result, version) = IPy.parseAddress('0x100000000')
         self.assertEqual(version, 6)
-            
+
     def testEmpty(self):
         """'' should raise an exception"""
         self.assertRaises(ValueError, IPy.parseAddress, '')
@@ -146,29 +148,29 @@ class _intToIP(unittest.TestCase):
                 (0xf, '0.0.0.15'),
                 (0xff, '0.0.0.255'),
                 (0xFFFFFFFFL, '255.255.255.255')]
-    v6values = [(0x7f000001, '0000:0000:0000:0000:0000:0000:7F00:0001'),
+    v6values = [(0x7f000001, '0000:0000:0000:0000:0000:0000:7f00:0001'),
                 (0x0, '0000:0000:0000:0000:0000:0000:0000:0000'),
                 (0x1, '0000:0000:0000:0000:0000:0000:0000:0001'),
-                (0xf, '0000:0000:0000:0000:0000:0000:0000:000F'),
-                (0xff, '0000:0000:0000:0000:0000:0000:0000:00FF'),
-                (0xFFFFFFFFL, '0000:0000:0000:0000:0000:0000:FFFF:FFFF'),
-                (0x100000000L, '0000:0000:0000:0000:0000:0001:0000:0000'),
-                (0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFL, 'FFFF:FFFF:FFFF:FFFF:FFFF:FFFF:FFFF:FFFF')]
+                (0xf, '0000:0000:0000:0000:0000:0000:0000:000f'),
+                (0xff, '0000:0000:0000:0000:0000:0000:0000:00ff'),
+                (0xFFFFFFFFL, '0000:0000:0000:0000:0000:0000:ffff:ffff'),
+                (0x100000000, '0000:0000:0000:0000:0000:0001:0000:0000'),
+                (0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFL, 'ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff')]
 
     def testKnownValuesv4(self):
         """printing of known IPv4 values should give known results"""
         for x in self.v4values:
             (question, answer) = x
-            result  = IPy.intToIp(question, 4)
+            result  = IPy.intToIp(question, 4).lower()
             self.assertEqual(answer, result, "%r, %r, %r" % (question, answer, result))
 
     def testKnownValuesv6(self):
         """printing of known IPv6 values should give known results"""
         for x in self.v6values:
             (question, answer) = x
-            result  = IPy.intToIp(question, 6)
+            result  = IPy.intToIp(question, 6).lower()
             self.assertEqual(answer, result, "%r, %r, %r" % (question, answer, result))
- 
+
     def testNegativeIPv4(self):
         """negative IPv4 Values should raise an exception"""
         self.assertRaises(ValueError, IPy.intToIp, -1, 4)
@@ -179,11 +181,11 @@ class _intToIP(unittest.TestCase):
 
     def testLargeIPv4(self):
         """IPv4: Values > 0xffffffff should raise an exception"""
-        self.assertRaises(ValueError, IPy.intToIp, 0x100000000L, 4)
+        self.assertRaises(ValueError, IPy.intToIp, 0x100000000, 4)
 
     def testLargeIPv6(self):
         """IPv6: Values > 0xffffffffffffffffffffffffffffffff should raise an exception"""
-        self.assertRaises(ValueError, IPy.intToIp, 0x100000000000000000000000000000000L, 6)
+        self.assertRaises(ValueError, IPy.intToIp, 0x100000000000000000000000000000000, 6)
 
     def testIllegalVersion(self):
         """IPVersion other than 4 and 6 should raise an exception"""
@@ -208,8 +210,8 @@ class ParseAndBack(unittest.TestCase):
                         ((long(random.randrange(0x7fffffff)) + long(random.randrange(0x7fffffff))) << 64) +
                         ((long(random.randrange(0x7fffffff)) + long(random.randrange(0x7fffffff))) << 96))
             self.assertEqual(IPy.parseAddress(IPy.intToIp(question, 6)), (question, 6), hex(question))
-        
-        
+
+
 class _countXBits(unittest.TestCase):
     def testCount1Bits(self):
         self.assertEqual(IPy._count1Bits(0), 0)
@@ -222,28 +224,28 @@ class _countXBits(unittest.TestCase):
 
     def testCount1Bits(self):
         self.assertEqual(IPy._count0Bits(0), 0)
-        self.assertEqual(IPy._count0Bits(0xf0L), 4)
-        self.assertEqual(IPy._count0Bits(0xf00L), 8)
-        self.assertEqual(IPy._count0Bits(0xf000L), 12)
-        self.assertEqual(IPy._count0Bits(0xf0000L), 16)
-        self.assertEqual(IPy._count0Bits(0xf00000L), 20)
-        self.assertEqual(IPy._count0Bits(0xf000000L), 24)
-        self.assertEqual(IPy._count0Bits(0xf0000000L), 28)
-        self.assertEqual(IPy._count0Bits(0xff000000L), 24)
-        self.assertEqual(IPy._count0Bits(0xfff00000L), 20)
-        self.assertEqual(IPy._count0Bits(0x80000000L), 31)
-        self.assertEqual(IPy._count0Bits(0xf0000000000000000000000000000000L), 124)
-        self.assertEqual(IPy._count0Bits(0x80000000000000000000000000000000L), 127)
-        
-       
+        self.assertEqual(IPy._count0Bits(0xf0), 4)
+        self.assertEqual(IPy._count0Bits(0xf00), 8)
+        self.assertEqual(IPy._count0Bits(0xf000), 12)
+        self.assertEqual(IPy._count0Bits(0xf0000), 16)
+        self.assertEqual(IPy._count0Bits(0xf00000), 20)
+        self.assertEqual(IPy._count0Bits(0xf000000), 24)
+        self.assertEqual(IPy._count0Bits(0xf0000000), 28)
+        self.assertEqual(IPy._count0Bits(0xff000000), 24)
+        self.assertEqual(IPy._count0Bits(0xfff00000), 20)
+        self.assertEqual(IPy._count0Bits(0x80000000), 31)
+        self.assertEqual(IPy._count0Bits(0xf0000000000000000000000000000000), 124)
+        self.assertEqual(IPy._count0Bits(0x80000000000000000000000000000000), 127)
+
+
 class _intToBin(unittest.TestCase):
     knownValues = [(0, '0'), (1, '1'), (2, '10'), (3, '11'), (4, '100'), (5, '101'),
                    (6, '110'), (7, '111'), (8, '1000'), (9, '1001'),
                    (0xf, '1111'), (0xff, '11111111'),
                    (0xFFFFFFFFL, '11111111111111111111111111111111'),
-                   (0x100000000L, '100000000000000000000000000000000'),
+                   (0x100000000, '100000000000000000000000000000000'),
                    (0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFL, '11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111'),
-                   (0x100000000000000000000000000000000L, '100000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000')]
+                   (0x100000000000000000000000000000000, '100000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000')]
 
     def testKnownValues(self):
         """conversion of known values values should give known results"""
@@ -257,28 +259,28 @@ class _intToBin(unittest.TestCase):
         self.assertRaises(ValueError, IPy._intToBin, -1)
 
 class netmaskPrefixlenConv(unittest.TestCase):
-    known4Values = [(0xFFFFFFFFL, 32), (0xFFFFFFFEL, 31), (0xFFFFFFFCL, 30), (0xFFFFFFF8L, 29),
-                    (0xFFFFFFF0L, 28), (0xFFFFFFE0L, 27), (0xFFFFFFC0L, 26), (0xFFFFFF80L, 25),
-                    (0xFFFFFF00L, 24), (0xFFFFFE00L, 23), (0xFFFFFC00L, 22), (0xFFFFF800L, 21),
-                    (0xFFFFF000L, 20), (0xFFFFE000L, 19), (0xFFFFC000L, 18), (0xFFFF8000L, 17),
-                    (0xFFFF0000L, 16), (0xFFFE0000L, 15), (0xFFFC0000L, 14), (0xFFF80000L, 13),
-                    (0xFFF00000L, 12), (0xFFE00000L, 11), (0xFFC00000L, 10), (0xFF800000L, 9),
-                    (0xFF000000L, 8), (0xFE000000L, 7), (0xFC000000L, 6), (0xF8000000L, 5),
-                    (0xF0000000L, 4), (0xE0000000L, 3), (0xC0000000L, 2), (0x80000000L, 1)]
+    known4Values = [(0xFFFFFFFFL, 32), (0xFFFFFFFEL, 31), (0xFFFFFFFCL, 30), (0xFFFFFFF8, 29),
+                    (0xFFFFFFF0, 28), (0xFFFFFFE0, 27), (0xFFFFFFC0, 26), (0xFFFFFF80, 25),
+                    (0xFFFFFF00, 24), (0xFFFFFE00, 23), (0xFFFFFC00, 22), (0xFFFFF800, 21),
+                    (0xFFFFF000, 20), (0xFFFFE000, 19), (0xFFFFC000, 18), (0xFFFF8000, 17),
+                    (0xFFFF0000, 16), (0xFFFE0000, 15), (0xFFFC0000, 14), (0xFFF80000, 13),
+                    (0xFFF00000, 12), (0xFFE00000, 11), (0xFFC00000, 10), (0xFF800000, 9),
+                    (0xFF000000, 8), (0xFE000000, 7), (0xFC000000, 6), (0xF8000000, 5),
+                    (0xF0000000, 4), (0xE0000000, 3), (0xC0000000, 2), (0x80000000, 1)]
     known6Values = [(0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFL, 128),
                     (0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEL, 127),
-                    (0xFFFFFFFFFFFFFFFFFFFFFFFF80000000L, 97),
-                    (0xFFFFFFFFFFFFFFFFFFFFFFFF00000000L, 96),
-                    (0xFFFFFFFFFFFFFFFFFFFFFFFE00000000L, 95),
-                    (0xFFFFFFFFFFFFFFFF8000000000000000L, 65),
-                    (0xFFFFFFFFFFFFFFFF0000000000000000L, 64),
-                    (0xFFFFFFFFFFFFFFFE0000000000000000L, 63),
-                    (0xFFFFFFFF800000000000000000000000L, 33),
-                    (0xFFFFFFFF000000000000000000000000L, 32),
-                    (0xFFFFFFFE000000000000000000000000L, 31),
-                    (0xC0000000000000000000000000000000L, 2),
-                    (0x80000000000000000000000000000000L, 1)]
-    
+                    (0xFFFFFFFFFFFFFFFFFFFFFFFF80000000, 97),
+                    (0xFFFFFFFFFFFFFFFFFFFFFFFF00000000, 96),
+                    (0xFFFFFFFFFFFFFFFFFFFFFFFE00000000, 95),
+                    (0xFFFFFFFFFFFFFFFF8000000000000000, 65),
+                    (0xFFFFFFFFFFFFFFFF0000000000000000, 64),
+                    (0xFFFFFFFFFFFFFFFE0000000000000000, 63),
+                    (0xFFFFFFFF800000000000000000000000, 33),
+                    (0xFFFFFFFF000000000000000000000000, 32),
+                    (0xFFFFFFFE000000000000000000000000, 31),
+                    (0xC0000000000000000000000000000000, 2),
+                    (0x80000000000000000000000000000000, 1)]
+
     def testKnownValuesv4n2p(self):
         """conversion of known values values should give known results"""
         for x in self.known4Values:
@@ -309,11 +311,11 @@ class netmaskPrefixlenConv(unittest.TestCase):
 
     def testInvalidv4n2p(self):
         """Netmasks should be all ones in the first part and all zeros in the second part"""
-        self.failUnlessRaises(ValueError, IPy._netmaskToPrefixlen, 0xff00ff00L)
+        self.failUnlessRaises(ValueError, IPy._netmaskToPrefixlen, 0xff00ff00)
 
     def testInvalidv6n2p(self):
         """Netmasks should be all ones in the first part and all zeros in the second part"""
-        self.failUnlessRaises(ValueError, IPy._netmaskToPrefixlen, 0xff00ff00ff00ff00ff00ff00ff00ff00L)
+        self.failUnlessRaises(ValueError, IPy._netmaskToPrefixlen, 0xff00ff00ff00ff00ff00ff00ff00ff00)
 
 
 class checkChecks(unittest.TestCase):
@@ -321,59 +323,59 @@ class checkChecks(unittest.TestCase):
     def testCheckNetmaskOk(self):
         """Legal Netmasks should be allowed."""
         self.failIf(IPy._checkNetmask(0xffffffffL, 32))
-        self.failIf(IPy._checkNetmask(0xffffff00L, 32))
-        self.failIf(IPy._checkNetmask(0xffff0000L, 32))
-        self.failIf(IPy._checkNetmask(0xff000000L, 32))
+        self.failIf(IPy._checkNetmask(0xffffff00, 32))
+        self.failIf(IPy._checkNetmask(0xffff0000, 32))
+        self.failIf(IPy._checkNetmask(0xff000000, 32))
         self.failIf(IPy._checkNetmask(0, 32))
 
     def testCheckNetmaskFail(self):
         """Illegal Netmasks should be rejected."""
         self.failUnlessRaises(ValueError, IPy._checkNetmask, 0xf0ffffffL, 32)
-        self.failUnlessRaises(ValueError, IPy._checkNetmask, 0xf0f0f0f0L, 32)
-        self.failUnlessRaises(ValueError, IPy._checkNetmask, 0xff00ff00L, 32)
-        self.failUnlessRaises(ValueError, IPy._checkNetmask, 0x70000001L, 32)
+        self.failUnlessRaises(ValueError, IPy._checkNetmask, 0xf0f0f0f0, 32)
+        self.failUnlessRaises(ValueError, IPy._checkNetmask, 0xff00ff00, 32)
+        self.failUnlessRaises(ValueError, IPy._checkNetmask, 0x70000001, 32)
         self.failUnlessRaises(ValueError, IPy._checkNetmask, 0xfffffffL, 32)
 
     def testCheckPrefixOk(self):
         """Legal IP/prefix combinations should check ok."""
         self.failUnless(IPy._checkPrefix(0x0, 32, 4))
         self.failUnless(IPy._checkPrefix(0xffffffffL, 32, 4))
-        self.failUnless(IPy._checkPrefix(0x7f000001L, 32, 4))
-        self.failUnless(IPy._checkPrefix(0x80000000L, 1, 4))
-        self.failUnless(IPy._checkPrefix(0x40000000L, 2, 4))
-        self.failUnless(IPy._checkPrefix(0x80000000L, 3, 4))
-        self.failUnless(IPy._checkPrefix(0x80000000L, 4, 4))
-        self.failUnless(IPy._checkPrefix(0xffffff00L, 24, 4))
-        self.failUnless(IPy._checkPrefix(0xffffff00L, 24, 4))
-        self.failUnless(IPy._checkPrefix(0xfffffff0L, 28, 4))
+        self.failUnless(IPy._checkPrefix(0x7f000001, 32, 4))
+        self.failUnless(IPy._checkPrefix(0x80000000, 1, 4))
+        self.failUnless(IPy._checkPrefix(0x40000000, 2, 4))
+        self.failUnless(IPy._checkPrefix(0x80000000, 3, 4))
+        self.failUnless(IPy._checkPrefix(0x80000000, 4, 4))
+        self.failUnless(IPy._checkPrefix(0xffffff00, 24, 4))
+        self.failUnless(IPy._checkPrefix(0xffffff00, 24, 4))
+        self.failUnless(IPy._checkPrefix(0xfffffff0, 28, 4))
         self.failUnless(IPy._checkPrefix(0x0, 32, 4))
         self.failUnless(IPy._checkPrefix(0x0, 1, 4))
         self.failUnless(IPy._checkPrefix(0x0, 0, 4))
-        self.failUnless(IPy._checkPrefix(0xffffffffffffffff0000000000000000L, 64, 6))
-        self.failUnless(IPy._checkPrefix(0x0L, 64, 6))
-        self.failUnless(IPy._checkPrefix(0x0L, 0, 6))
-        self.failUnless(IPy._checkPrefix(0x0L, 128, 6))
+        self.failUnless(IPy._checkPrefix(0xffffffffffffffff0000000000000000, 64, 6))
+        self.failUnless(IPy._checkPrefix(0x0, 64, 6))
+        self.failUnless(IPy._checkPrefix(0x0, 0, 6))
+        self.failUnless(IPy._checkPrefix(0x0, 128, 6))
         self.failUnless(IPy._checkPrefix(0xffffffffffffffffffffffffffffffffL, 128, 6))
 
 
     def testCheckPrefixFail(self):
         """Illegal Prefixes should be catched."""
-        self.failIf(IPy._checkPrefix(0x7f000001L, -1, 4))
-        self.failIf(IPy._checkPrefix(0x7f000001L, 33, 4))
-        self.failIf(IPy._checkPrefix(0x7f000001L, 24, 4))
-        self.failIf(IPy._checkPrefix(0x7f000001L, 31, 4))
-        self.failIf(IPy._checkPrefix(0x7f000080L, 24, 4))
-        self.failIf(IPy._checkPrefix(0x7f000100L, 23, 4))
-        self.failIf(IPy._checkPrefix(0x7f000000L, 1, 4))
-        self.failIf(IPy._checkPrefix(0x7f000000L, 0, 4))
-        self.failIf(IPy._checkPrefix(0x1L, -1, 6))
-        self.failIf(IPy._checkPrefix(0x1L, 129, 6))
-        self.failIf(IPy._checkPrefix(0xffffffffffffffff0000000000000001L, 64, 6))
-        self.failIf(IPy._checkPrefix(0xffffffffffffffff1000000000000000L, 64, 6))
-    
+        self.failIf(IPy._checkPrefix(0x7f000001, -1, 4))
+        self.failIf(IPy._checkPrefix(0x7f000001, 33, 4))
+        self.failIf(IPy._checkPrefix(0x7f000001, 24, 4))
+        self.failIf(IPy._checkPrefix(0x7f000001, 31, 4))
+        self.failIf(IPy._checkPrefix(0x7f000080, 24, 4))
+        self.failIf(IPy._checkPrefix(0x7f000100, 23, 4))
+        self.failIf(IPy._checkPrefix(0x7f000000, 1, 4))
+        self.failIf(IPy._checkPrefix(0x7f000000, 0, 4))
+        self.failIf(IPy._checkPrefix(0x1, -1, 6))
+        self.failIf(IPy._checkPrefix(0x1, 129, 6))
+        self.failIf(IPy._checkPrefix(0xffffffffffffffff0000000000000001, 64, 6))
+        self.failIf(IPy._checkPrefix(0xffffffffffffffff1000000000000000, 64, 6))
+
 
     # TODO: _checkNetaddrWorksWithPrefixlen(net, prefixlen, version):
-    
+
 class PythonObjectBehaviour(unittest.TestCase):
     def testIfUsuableAsDictionaryKey(self):
         """IP Object should be usable as dictionary key"""
@@ -433,7 +435,7 @@ class PythonObjectBehaviour(unittest.TestCase):
         ip = IPy.IP('2001::/124')
         for x in ip:
             self.failUnless(x.strNormal() in ip, "%r not in %r" % (x.strNormal(), ip))
- 
+
     def testIfContainsIPobj(self):
         """__contains__() should work somewhat with IP instances"""
         ip = IPy.IP('127.0.0.0/28')
@@ -450,7 +452,7 @@ class PythonObjectBehaviour(unittest.TestCase):
         self.assertEqual(ip[-1], ip.broadcast())
         self.failUnless(ip[255])
         self.failUnlessRaises(IndexError, ip.__getitem__, 256)
-        
+
     def testStr(self):
         """string() should work somewhat with IP instances"""
         ip = IPy.IP('127.0.0.0/28')
@@ -475,7 +477,7 @@ class PythonObjectBehaviour(unittest.TestCase):
         self.failUnlessEqual(len(IPy.IP('127.0.0.0/30')), 4)
         self.failUnlessEqual(len(IPy.IP('127.0.0.0/26')), 64)
         self.failUnlessEqual(len(IPy.IP('127.0.0.0/16')), 2**16)
-        
+
     # cmp
     # IP[0xffffffff]
     # IP + IP
@@ -509,7 +511,7 @@ class IPobject(unittest.TestCase):
                   '1:0:0:2::3',
                   '1::2:0:0:3']
         for question in testValues:
-            result = IPy.IP(question).strCompressed() 
+            result = IPy.IP(question).strCompressed()
             self.failUnlessEqual(question, result, (question, result))
 
     def testStrBin(self):
@@ -529,71 +531,71 @@ class IPobject(unittest.TestCase):
                       ('127.0.0.1', '01111111000000000000000000000001'),
                       ('1::2:0:0:3', '00000000000000010000000000000000000000000000000000000000000000000000000000000010000000000000000000000000000000000000000000000011')]
         for (question, answer) in testValues:
-            result = IPy.IP(question).strBin() 
+            result = IPy.IP(question).strBin()
             self.failUnlessEqual(answer, result, (question, answer, result))
 
     def testStrNormal(self):
         """Normal string Output."""
-        testValues = [(338770000845734292534325025077361652240L, 'fedc:ba98:7654:3210:fedc:ba98:7654:3210'),
-                      (21932261930451111902915077091070067066L, '1080:0:0:0:8:800:200c:417a'),
-                      (338958331222012082418099330867817087043L, 'ff01:0:0:0:0:0:0:43'),
-                      (0L, '0.0.0.0'),
-                      (2130706433L, '127.0.0.1'),
-                      (4294967295L, '255.255.255.255'),
-                      (1L, '0.0.0.1'),
-                      (3588059479L, '213.221.113.87')]
+        testValues = [(338770000845734292534325025077361652240, 'fedc:ba98:7654:3210:fedc:ba98:7654:3210'),
+                      (21932261930451111902915077091070067066, '1080:0:0:0:8:800:200c:417a'),
+                      (338958331222012082418099330867817087043, 'ff01:0:0:0:0:0:0:43'),
+                      (0, '0.0.0.0'),
+                      (2130706433, '127.0.0.1'),
+                      (4294967295, '255.255.255.255'),
+                      (1, '0.0.0.1'),
+                      (3588059479, '213.221.113.87')]
         for (question, answer) in testValues:
-            result = IPy.IP(question).strNormal(question) 
+            result = IPy.IP(question).strNormal(question)
             self.failUnlessEqual(answer, result, (question, result, answer))
 
     def testStrFullsize(self):
         """Normal / 0-padded string Output."""
-        testValues = [(338770000845734292534325025077361652240L, 'fedc:ba98:7654:3210:fedc:ba98:7654:3210'),
-                      (21932261930451111902915077091070067066L, '1080:0000:0000:0000:0008:0800:200c:417a'),
-                      (338958331222012082418099330867817087043L, 'ff01:0000:0000:0000:0000:0000:0000:0043'),
-                      (0L, '0.0.0.0'),
-                      (2130706433L, '127.0.0.1'),
-                      (4294967295L, '255.255.255.255'),
-                      (1L, '0.0.0.1'),
-                      (3588059479L, '213.221.113.87')]
+        testValues = [(338770000845734292534325025077361652240, 'fedc:ba98:7654:3210:fedc:ba98:7654:3210'),
+                      (21932261930451111902915077091070067066, '1080:0000:0000:0000:0008:0800:200c:417a'),
+                      (338958331222012082418099330867817087043, 'ff01:0000:0000:0000:0000:0000:0000:0043'),
+                      (0, '0.0.0.0'),
+                      (2130706433, '127.0.0.1'),
+                      (4294967295, '255.255.255.255'),
+                      (1, '0.0.0.1'),
+                      (3588059479, '213.221.113.87')]
         for (question, answer) in testValues:
-            result = IPy.IP(question).strFullsize(question) 
+            result = IPy.IP(question).strFullsize(question)
             self.failUnlessEqual(answer, result, (question, result, answer))
 
     def testStrHex(self):
         """Hex string Output."""
-        testValues = [(338770000845734292534325025077361652240L, '0xFEDCBA9876543210FEDCBA9876543210'),
-                      (21932261930451111902915077091070067066L, '0x108000000000000000080800200C417A'),
-                      (338958331222012082418099330867817087043L, '0xFF010000000000000000000000000043'),
-                      (0L, '0x0'),
-                      (1L, '0x1'),
-                      (4294967295l, '0xFFFFFFFF'),
-                      (3588059479L, '0xD5DD7157'),
+        testValues = [(338770000845734292534325025077361652240, '0xfedcba9876543210fedcba9876543210'),
+                      (21932261930451111902915077091070067066, '0x108000000000000000080800200c417a'),
+                      (338958331222012082418099330867817087043, '0xff010000000000000000000000000043'),
+                      (0, '0x0'),
+                      (1, '0x1'),
+                      (4294967295l, '0xffffffff'),
+                      (3588059479, '0xd5dd7157'),
                       (0x12345678, '0x12345678')]
         for (question, answer) in testValues:
-            result = IPy.IP(question).strHex(question) 
+            result = IPy.IP(question).strHex(question).lower()
             self.failUnlessEqual(answer, result, (question, result, answer))
 
     def testStrDec(self):
         """Decimal string Output."""
-        testValues = [(338770000845734292534325025077361652240L, '338770000845734292534325025077361652240'),
-                      (21932261930451111902915077091070067066L, '21932261930451111902915077091070067066'),
-                      (338958331222012082418099330867817087043L, '338958331222012082418099330867817087043'),
-                      (0L, '0'),
-                      (1L, '1'),
+        testValues = [(338770000845734292534325025077361652240, '338770000845734292534325025077361652240'),
+                      (21932261930451111902915077091070067066, '21932261930451111902915077091070067066'),
+                      (338958331222012082418099330867817087043, '338958331222012082418099330867817087043'),
+                      (0, '0'),
+                      (1, '1'),
                       (0xFFFFFFFFL, '4294967295'),
-                      (0xD5DD7157L, '3588059479')]
+                      (0xD5DD7157, '3588059479')]
         for (question, answer) in testValues:
-            result = IPy.IP(question).strDec(question) 
+            result = IPy.IP(question).strDec(question)
             self.failUnlessEqual(answer, result, (question, result, answer))
 
     def testNet(self):
         """Returning of the Network Address"""
         self.failUnlessEqual(str(IPy.IP("127.0.0.1").net()), "127.0.0.1")
         self.failUnlessEqual(str(IPy.IP("0.0.0.0/0").net()), "0.0.0.0")
-        self.failUnlessEqual(str(IPy.IP("2001:1234:5678:1234::/64").net()), "2001:1234:5678:1234:0000:0000:0000:0000")
+        self.failUnlessEqual(str(IPy.IP("2001:1234:5678:1234::/64").net()), "2001:1234:5678:1234::")
+
 
-        
     def testBroadcast(self):
         """Returning of broadcast address."""
         self.failUnlessEqual(str(IPy.IP("127.0.0.1").broadcast()), "127.0.0.1")
@@ -623,8 +625,8 @@ class IPobject(unittest.TestCase):
         self.failUnlessEqual(IPy.IP("255.255.255.255").int(), 0xffffffffL)
         self.failUnlessEqual(IPy.IP("0000:0000:0000:0000:0000:0000:0000:0000").int(), 0)
         self.failUnlessEqual(IPy.IP("ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff").int(), 0xffffffffffffffffffffffffffffffffL)
-        self.failUnlessEqual(IPy.IP("2001:1234:5678:9abc:de00:0000:0000:0000").int(), 42540857391974671903776007410583339008L)
-    
+        self.failUnlessEqual(IPy.IP("2001:1234:5678:9abc:de00:0000:0000:0000").int(), 42540857391974671903776007410583339008)
+
 
     def testPrefixlen(self):
         """Prefixlen"""
@@ -635,7 +637,7 @@ class IPobject(unittest.TestCase):
         self.failUnlessEqual(IPy.IP("10.0.0.0/255.255.255.0").prefixlen(), 24)
         self.failUnlessEqual(IPy.IP("2001::/64").prefixlen(), 64)
 
-    
+
     def testVersion(self):
         """IP-version detection should work"""
         self.failUnlessEqual(IPy.IP("0.0.0.0/0").version(), 4)
@@ -648,21 +650,36 @@ class IPobject(unittest.TestCase):
     #def __add__(self, other):
     #def _printPrefix(self, want):
 
+    def testOverlaps(self):
+        """Overlapping Address Ranges."""
+        testValues = [('192.168.0.0/23', '192.168.1.0/24', 1),
+                      ('192.168.0.0/23', '192.168.0.0/20', 1),
+                      ('192.168.0.0/23', '192.168.2.0', 0),
+                      ('192.168.0.0/23', '192.167.255.255', 0),
+                      ('192.168.0.0/23', '192.168.0.0', 1),
+                      ('192.168.0.0/23', '192.168.1.255', 1),
+                      ('192.168.1.0/24', '192.168.0.0/23', -1),
+                      ('127.0.0.1', '127.0.0.1', 1),
+                      ('127.0.0.1', '127.0.0.2', 0)]
+        for (a, b, answer) in testValues:
+            result = IPy.IP(a).overlaps(b)
+            self.failUnlessEqual(answer, result, (a, b, result, answer))
+
     def testNetmask(self):
         """Normal string Output."""
-        testValues = [(338770000845734292534325025077361652240L, '0xFEDCBA9876543210FEDCBA9876543210'),
-                      (21932261930451111902915077091070067066L, '0x108000000000000000080800200C417A'),
-                      (338958331222012082418099330867817087043L, '0xFF010000000000000000000000000043'),
-                      (0L, '0x0'),
-                      (1L, '0x1'),
-                      (4294967295l, '0xFFFFFFFF'),
-                      (3588059479L, '0xD5DD7157')]
+        testValues = [(338770000845734292534325025077361652240, '0xfedcba9876543210fedcba9876543210'),
+                      (21932261930451111902915077091070067066, '0x108000000000000000080800200c417a'),
+                      (338958331222012082418099330867817087043, '0xff010000000000000000000000000043'),
+                      (0, '0x0'),
+                      (1, '0x1'),
+                      (4294967295l, '0xffffffff'),
+                      (3588059479, '0xd5dd7157')]
         for (question, answer) in testValues:
-            result = IPy.IP(question).strHex(question) 
+            result = IPy.IP(question).strHex(question).lower()
             self.failUnlessEqual(answer, result, (question, result, answer))
 
 # TODO
-#eval(repr(IPy)) 
+#eval(repr(IPy))
 # differences between IP and IPint
 
 
@@ -671,7 +688,7 @@ class NetIPChecks(unittest.TestCase):
     """Checks taken from perls Net::IP"""
     def testMisc(self):
         ip = IPy.IP('195.114.80/24')
-        self.assertEqual(ip.int(), 3279048704L)
+        self.assertEqual(ip.int(), 3279048704)
         self.assertEqual(ip.reverseName(),'80.114.195.in-addr.arpa.')
         self.assertEqual(ip.strBin(),'11000011011100100101000000000000')
         self.assertEqual(str(ip.net()),'195.114.80.0')
@@ -687,43 +704,43 @@ class NetIPChecks(unittest.TestCase):
 
         ip = IPy.IP('202.31.4/24')
         self.assertEqual(str(ip.net()),'202.31.4.0')
-        
-        self.failUnlessRaises(ValueError, IPy.IP, '234.245.252.253/2')        
+
+        self.failUnlessRaises(ValueError, IPy.IP, '234.245.252.253/2')
 
         # because we ar using integer representation we don't need a special "binadd"
         ip = IPy.IP('62.33.41.9')
         ip2 = IPy.IP('0.1.0.5')
         self.assertEqual(str(IPy.IP(ip.int() + ip2.int())),'62.34.41.14')
         #$T->ok_eq ($ip->binadd($ip2)->ip(),'62.34.41.14',$ip->error());
-        
+
         ip = IPy.IP('133.45.0/24')
         ip2 = IPy.IP('133.45.1/24')
         self.assertEqual((ip + ip2).prefixlen(),23)
-        
+
         ip2 = IPy.IP('133.44.255.255');
         #$T->ok_eqnum ($ip->bincomp('gt',$ip2),1,$ip->error());
 
         # this is something we can't do with IPy
         #ip = IPy.IP('133.44.255.255-133.45.0.42');
         #$T->ok_eq (($ip->find_prefixes())[3],'133.45.0.40/31',$ip->error());
-        
+
         ip = IPy.IP('201.33.128.0/22');
         ip2 = IPy.IP('201.33.129.0/24');
         #$T->ok_eqnum ($ip->overlaps($ip2),$IP_B_IN_A_OVERLAP,$ip->error());
-        
+
         ip = IPy.IP('dead:beef:0::/48')
-        self.assertEqual(str(ip.net()),'dead:beef:0000:0000:0000:0000:0000:0000')
-        self.assertEqual(ip.int(), 295990755014133383690938178081940045824L)
+        self.assertEqual(str(ip.net()),'dead:beef::')
+        self.assertEqual(ip.int(), 295990755014133383690938178081940045824)
         self.assertEqual(ip.strBin(),'11011110101011011011111011101111000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000')
         self.assertEqual(ip.strCompressed(),'dead:beef::/48')
         self.assertEqual(ip.prefixlen(), 48)
         self.assertEqual(ip.version(), 6)
         self.assertEqual(ip.strNetmask(),'/48')
-        self.assertEqual(str(ip.netmask()),'ffff:ffff:ffff:0000:0000:0000:0000:0000')
-        self.assertEqual(ip.iptype(),'UNASSIGNED')
-        self.assertEqual(ip.reverseName(),'0.0.0.0.f.e.e.b.d.a.e.d.ip6.int.')
-        self.assertEqual(str(ip.broadcast()),'dead:beef:0000:ffff:ffff:ffff:ffff:ffff')
-        
+        self.assertEqual(str(ip.netmask()),'ffff:ffff:ffff::')
+        self.assertEqual(ip.iptype(),'RESERVED')
+        self.assertEqual(ip.reverseName(),'0.0.0.0.f.e.e.b.d.a.e.d.ip6.arpa.')
+        self.assertEqual(str(ip.broadcast()),'dead:beef:0:ffff:ffff:ffff:ffff:ffff')
+
         ip = IPy.IP('202.31.4/24')
         self.assertEqual(str(ip.net()),'202.31.4.0')
 
@@ -735,21 +752,84 @@ class NetIPChecks(unittest.TestCase):
         ip = IPy.IP('ff00:0:f000::')
         ip2 = IPy.IP('0:0:1000::')
         self.assertEqual(IPy.IP(ip.int() + ip2.int()).strCompressed(), 'ff00:1::')
-    
+
         ip = IPy.IP('::e000:0/112')
         ip2 = IPy.IP('::e001:0/112')
         self.assertEqual(ip.__add__(ip2).prefixlen(),111)
-        
+
         ip2 = IPy.IP('::dfff:ffff')
         #$T->ok_eqnum ($ip->bincomp('gt',$ip2),1,$ip->error());
-        
+
         #ip = IPy.IP('::e000:0 - ::e002:42')
         #$T->ok_eq (($ip->find_prefixes())[2],'0000:0000:0000:0000:0000:0000:e002:0040/127',$ip->error());
-        
+
         ip = IPy.IP('ffff::/16')
         ip2 = IPy.IP('8000::/16')
         #$T->ok_eqnum ($ip->overlaps($ip2),$IP_NO_OVERLAP,$ip->error());
 
+def timeout(func, args=(), kwargs={}, timeout_duration=1, default=None):
+    """
+    ASPN receipe written by dustin lee to call a function with
+    a timeout using threads:
+    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/473878
+
+    Small patch: add setDaemon(True) to allow Python to leave whereas the
+    thread is not done.
+    """
+    class InterruptableThread(threading.Thread):
+        def __init__(self):
+            threading.Thread.__init__(self)
+            self.result = None
+
+        def run(self):
+            try:
+                self.result = func(*args, **kwargs)
+            except:
+                self.result = default
+
+    it = InterruptableThread()
+    it.setDaemon(True)
+    it.start()
+    it.join(timeout_duration)
+    if it.isAlive():
+        return default
+    else:
+        return it.result
+
+class RegressionTest(unittest.TestCase):
+    def testNulNetmask(self):
+        ip = timeout(IPy.IP, ["0.0.0.0/0.0.0.0"], timeout_duration=0.250, default=None)
+        if ip:
+            text = str(ip)
+        else:
+            text = "*TIMEOUT*"
+        self.assertEqual(text, "0.0.0.0/0")
+
+    def testNonZeroType(self):
+        self.assertEqual(bool(IPy.IP("0.0.0.0/0")), True)
+
+    def testPrivate169(self):
+        """
+        RFC 3330 indicates that 169.254.0.0/16 addresses are private.
+        They are automatically configured for links in the absence of other
+        information and should not be used on the internet
+        """
+        self.assertEqual(IPy.IP("169.254.191.164").iptype(), "PRIVATE")
+
+    def testCheckAddrPrefixlenOn(self):
+        self.assertEqual(len(IPy.IP('192.168.0.0/24')), 256)
+        self.assertRaises(ValueError, IPy.IP, '192.168.1.0/42')
+        self.assertRaises(ValueError, IPy.IP, '172.30.1.0/22')
+
+    def testCheckAddrPrefixlenOff(self):
+        self.assertEqual(len(IPy.IP('192.168.0.0/24')), 256)
+        self.assertRaises(ValueError, IPy.IP, '192.168.1.0/42')
+
+class TestConstrutor(unittest.TestCase):
+    def testCheckAddrPrefixlenOff(self):
+        self.assertRaises(ValueError, IPy.IP, 0xffffffff + 1, ipversion=4)
+        self.assertRaises(ValueError, IPy.IP, 0xffffffffffffffffffffffffffffffff + 1, ipversion=6)
+
 if __name__ == "__main__":
     unittest.main()
-                
+
